@@ -51,6 +51,7 @@ module Puppet
         package database for installed version(s), and can select
         which out of a set of available versions of a package to
         install if asked."
+    feature :version_ranges, "The provider can ensure version ranges."
     feature :holdable, "The provider is capable of placing packages on hold
         such that they are not automatically upgraded as a result of
         other package dependencies unless explicit action is taken by
@@ -61,6 +62,9 @@ module Puppet
       passed to the installer command."
     feature :uninstall_options, "The provider accepts options to be
       passed to the uninstaller command."
+    feature :disableable, "The provider can disable packages. This feature is used by specifying `disabled` as the
+      desired value for the package.",
+      :methods => [:disable]
     feature :supports_flavors, "The provider accepts flavors, which are specific variants of packages."
     feature :package_settings, "The provider accepts package_settings to be
       ensured for the given package. The meaning and format of these settings is
@@ -80,10 +84,12 @@ module Puppet
         specifying `purged` as the ensure value. This defaults to `installed`.
 
         Version numbers must match the full version to install, including
-        release if the provider uses a release moniker. Ranges or semver
-        patterns are not accepted except for the `gem` package provider. For
+        release if the provider uses a release moniker. For
         example, to install the bash package from the rpm
         `bash-4.1.2-29.el6.x86_64.rpm`, use the string `'4.1.2-29.el6'`.
+
+        On supported providers, version ranges can also be ensured. For example,
+        inequalities: `<2.0.0`, or intersections: `>1.0.0 <2.0.0`.
       EOT
 
       attr_accessor :latest
@@ -104,6 +110,10 @@ module Puppet
         provider.deprecated_hold
       end
 
+      newvalue(:disabled, :required_features => :disableable) do
+        provider.disable
+      end
+
       # Alias the 'present' value.
       aliasvalue(:installed, :present)
 
@@ -151,7 +161,7 @@ module Puppet
         @should.each { |should|
           case should
           when :present
-            return true unless [:absent, :purged, :held].include?(is)
+            return true unless [:absent, :purged, :held, :disabled].include?(is)
           when :latest
             # Short-circuit packages that are not present
             return false if is == :absent || is == :purged
@@ -275,6 +285,10 @@ module Puppet
     providify
     paramclass(:provider).isnamevar
 
+    def self.parameters_to_include
+      [:provider]
+    end
+
     # Specify a targeted package management command.
     newparam(:command, :required_features => :targetable) do
       desc <<-EOT
@@ -404,6 +418,11 @@ module Puppet
     newproperty(:flavor, :required_features => :supports_flavors) do
       desc "OpenBSD and DNF modules support 'flavors', which are
         further specifications for which type of package you want."
+      validate do |value|
+        if [:disabled, "disabled"].include?(@resource[:ensure]) && value
+          raise ArgumentError, _('Cannot have both `ensure => disabled` and `flavor`')
+        end
+      end
     end
 
     newparam(:source) do
@@ -485,6 +504,29 @@ module Puppet
       newvalues(:true, :false)
     end
 
+    newparam(:enable_only, :boolean => false, :parent => Puppet::Parameter::Boolean) do
+      desc <<-EOT
+        Tells `dnf module` to only enable a specific module, instead
+        of installing its default profile.
+
+        Modules with no default profile will be enabled automatically
+        without the use of this parameter.
+
+        Conflicts with the `flavor` property, which selects a profile
+        to install.
+      EOT
+      defaultto false
+
+      validate do |value|
+        if [true, :true, "true"].include?(value) && @resource[:flavor]
+          raise ArgumentError, _('Cannot have both `enable_only => true` and `flavor`')
+        end
+        if [:disabled, "disabled"].include?(@resource[:ensure])
+          raise ArgumentError, _('Cannot have both `ensure => disabled` and `enable_only => true`')
+        end
+      end
+    end
+
     newparam(:install_only, :boolean => false, :parent => Puppet::Parameter::Boolean, :required_features => :install_only) do
       desc <<-EOT
         It should be set for packages that should only ever be installed,
@@ -544,18 +586,27 @@ module Puppet
     newparam(:allow_virtual, :boolean => true, :parent => Puppet::Parameter::Boolean, :required_features => :virtual_packages) do
       desc 'Specifies if virtual package names are allowed for install and uninstall.'
 
-      defaultto true
+      defaultto do
+        provider_class = provider.class
+        if provider_class.respond_to?(:defaultto_allow_virtual)
+          provider_class.defaultto_allow_virtual
+        else
+          true
+        end
+      end
     end
 
     autorequire(:file) do
       autos = []
       [:responsefile, :adminfile].each { |param|
-        if val = self[param]
+        val = self[param]
+        if val
           autos << val
         end
       }
 
-      if source = self[:source] and absolute_path?(source)
+      source = self[:source]
+      if source && absolute_path?(source)
         autos << source
       end
       autos
@@ -563,7 +614,8 @@ module Puppet
 
     # This only exists for testing.
     def clear
-      if obj = @parameters[:ensure]
+      obj = @parameters[:ensure]
+      if obj
         obj.latest = nil
       end
     end
